{comb.dsp - FIR comb filter}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Eq.(8.2.8) of Introduction to Signal Processing.
 I/O equation: y(n) = a0 x(n) + a1 x(n-D) + a2 x(n-2D) + a3 x(n-3D)
 Sample processing algorithm:
       for each x do:
           *p = s0 = x                                  read filter input
           s1 = tap(3*D, w, p, D)                       get tap-D
           s2 = tap(3*D, w, p, 2*D)                     get tap-2D
           s3 = tap(3*D, w, p, 3*D)                     get tap-3D
           y = a0 * s0 + a1 * s1 + a2 * s2 + a3 * s3    filter output
           cdelay(3*D, w, &p)                           update delay
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const a0 = 0x7fff;                {a0 = 0.9999 = approximate unity}
.const a1 = 0x4000;                {a1 = 0.50}
.const a2 = 0x2000;                {a2 = a1^2 = 0.50^2 = 0.25}
.const a3 = 0x1000;                {a3 = a1^3 = 0.50^3 = 0.125}

.const D = 2000;                   {TD = D/fs = 2000/8000 = 1/4 sec}
.var/dm/circ w[3*D + 1];           {delay-line buffer, max delay = 3*D}

i2 = ^w;  L2 = %w;                 {delay-line buffer pointer and length}

zero(i2, m2, L2);                  {clear delay line}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   my1 = a0;
   mr = mx1 * my1 (ss);            {mr = a0 * x}

   tap(i2, m2, D, my0);            {my0 = s1 = D-th tap}
   mx0 = a1;
   mr = mr + mx0 * my0 (ss);       {mr = a0 * x + a1 * s1}

   tap(i2, m2, 2*D, my0);          {my0 = s2 = 2D-th tap}
   mx0 = a2;
   mr = mr + mx0 * my0 (ss);       {mr = a0 * x + a1 * s1 + a2 * s2}

   tap(i2, m2, 3*D, my0);          {my0 = s3 = 3D-th tap}
   mx0 = a3;
   mr = mr + mx0 * my0 (rnd);      {a0 * x + a1 * s1 + a2 * s2 + a3 * s3}
   if mv sat mr;                   {saturate mr if overflow}

   tapin(i2, m2, mx1);             {put input from mx1 into tap-0}
   cdelay(i2, m2);                 {update delay}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = mr1;           {left output sample}
   dm(tx_buf + 2) = mr1;           {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
