{fir3.dsp - FIR filtering of sinusoids generated by circular wavetable}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based of cfir.c and Sec. 8.1.3 of Introduction to Signal Processing}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const FS = 8000;                  {sampling rate in samples/sec}

.const M = 206;                    {from firlp.m with fc=200 Hz, Df=100 Hz}
.var/dm/circ w[M+1];               {circular delay-line buffer in DM}
.var/pm/circ h[M+1];               {filter coefficient buffer in PM}

.const D = 800;                    {basic frequency increment fs/D = 10 Hz}
.var/dm/circ s[D];                 {sinusoidal wavetable, from sinetbl.hex}
                                   {see Section 8.1.3 of text}
.const c1 = 10;                    {f1 = c1 * fs / D = c1 * 10 = 100 Hz}
.const c2 = 40;                    {f2 = c2 * fs / D = c2 * 10 = 400 Hz}
.const A1 = 0x4000;                {A1 = 0.5 = amplitude of sinusoid 1}
.const A2 = 0x4000;                {A2 = 0.5 = amplitude of sinusoid 2}

.const T = 4;                      {desired total duration in seconds}
.var/dm N;                         {N = fs * T = no. of samples in T sec}

ax1 = T * FS;                      {N = T * FS = duration in samples}
dm(N) = ax1;                       {save N in DM}

i2 = ^w; L2 = %w;                  {delay-line buffer pointer and length}
i4 = ^h; L4 = %h;                  {coefficient buffer pointer and length}

i5 = ^s; L5 = %s; m5 = c1;         {sinusoid f1, wavetable increment c1}
i6 = ^s; L6 = %s; m6 = c2;         {sinusoid f2, wavetable increment c2}

zero(i2, m2, L2);                  {clear delay line}

.init h: <fir.hex>;                {read coefficients in 1.15 hex format}
                                   {from file fir.hex}
.init s: <sinetbl.hex>;            {load sinusoidal wavetable}
                                   {from file sinetbl.hex}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- sample processing algorithm -----------------------------------------}

   ay0 = dm(N);
   ar = ay0 - 1;                          {decrement N until zero}
   dm(N) = ar;
   if eq jump stop;                       {do not stop until N iterations}

   mx1 = A1;
   my1 = dm(i5, m5);                      {my1 = s1 = sin(2*pi*f1*t)}
   mr = mx1 * my1 (ss);                   {mr = A1 * s1}

   mx1 = A2;
   my1 = dm(i6, m6);                      {my1 = s2 = sin(2*pi*f2*t)}
   mr = mr + mx1 * my1 (rnd);             {mr = x = A1 * s1 + A2 * s2}

#if 1                                     {use 0 to skip filtering}
   cfir(M, i4, m4, i2, m2, mr1);          {input from mr1, output in mr1}
#endif

{--- write samnples to codec ---------------------------------------------}

   dm(tx_buf + 1) = mr1;                  {left output sample}
   dm(tx_buf + 2) = mr1;                  {right output sample}

   rti;

stop:

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
