{flanger.dsp - flanging processor}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Eqs.(8.2.18-8.2.19) of Introduction to Signal Processing.
 I/O equation: y(n) = 0.5 * x(n) + 0.5 * x(n-d(n))
 Sample processing algorithm:
       for each x do:
           *p = s0 = x
           d = (D - D * sin(2*pi*fc*t)) / 2             variable delay
           s1 = tap(D, w, p, d)
           y = a0 * s0 + a1 * s1
           cdelay(D, w, &p)
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const a0 = 0x4000;                {a0 = 0.50}
.const a1 = 0x4000;                {a1 = 0.50}

.const D = 400;                    {TD = D/fs = 400/8000 = 50 msec}
.var/dm/circ w[D + 1];             {delay-line buffer, max delay = D}
.const D2 = D/2;

i2 = ^w;  L2 = %w;                 {delay-line buffer pointer and length}

zero(i2, m2, L2);                  {clear delay line}

.const Ds = 4000;                  {sinusoidal wavetable}
.var/dm/circ sine[Ds];             {min frequency f1 = fs/Ds = 8/4 = 2 Hz}

.init sine: <sinetbl.hex>;         {load one period of the wavetable}
                                   {generated by sinetbl.c}
.const c = 2;                      {signal frequency fc = c * f1 = 4 Hz}

i6 = ^sine; L6 = %sine;            {pointer for signal generator}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   wavgen(i6, m6, D2, c, ay1);     {ay1 = D2 * sin(2*pi*fc*t)}
   ax1 = D2;
   ar = ax1 - ay1;                 {ar = d = D2 - D2 * sin(2*pi*fc*t)}

   my1 = a0;
   mr = mx1 * my1 (ss);            {mr = a0 * x}

   m2 = ar; modify(i2, m2);        {these three lines replace the call}
   ar = -ar;                       {to tap(i2, m2, ar, my0), because}
   m2 = ar; my0 = dm(i2, m2);      {we cannot directly set m2 = -ar}

   mx0 = a1;
   mr = mr + mx0 * my0 (rnd);      {a0 * x + a * s1}
   if mv sat mr;

   tapin(i2, m2, mx1);             {put input from mx1 into tap-0}
   cdelay(i2, m2);                 {update delay}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = mr1;           {left output sample}
   dm(tx_buf + 2) = mr1;           {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
