{guitar.dsp - Karplus-Strong string algorithm}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Fig.8.2.21 and Eq.(8.2.40) of Introduction to Signal Processing.
 The filtering algorithm is similar to that of lowpass reverb, with the
 coefficient choices a = 0, b0 = b1 = 0.5. The length of the delay-line
 is designed by tuning it to a desired frequency, i.e., D = fs/f1.
 The buffer is filled with D+1 noise samples and the filter is run
 with zero input. Sample processing code is the same as in lowpass.dsp.
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc85b; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.var/dm v1;                        {state of first-order feedback filter}

.const a  = 0x0000;                {a  = 0}
.const b0 = 0x3fdf;                {b0 = 0.499}
.const b1 = 0x3fdf;                {b1 = 0.499}

.const D = 100;                    {f1 = fs/D = 44100/100 = 441 Hz}
.var/dm/circ w[D+1];               {circular delay line}

i2 = ^w;  L2 = %w;                 {delay-line buffer pointer and length}

.init w: <w.hex>;                  {load w with random numbers}
                                   {w.hex copied from w440hz.hex}
ax0 = 0;
dm(v1) = ax0;                      {clear feedback delay, v1 = 0}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

   mx1 = 0;                        {zero input}

{--- sample processing algorithm --- (from lowpass.dsp) ------------------}

   tap(i2, m2, D, mr1);                   {mr1 = sD = D-th tap}

   mx0 = a;
   my1 = dm(v1);                          {state v1}
   mr = mr + mx0 * my1 (rnd);             {mr = v0 = sD + a * v1}
   if mv sat mr;
   sr0 = mr1;                             {save v0 for later updating v1}

   my0 = b0;
   mr = mr1 * my0 (ss);                   {mr = b0 * v0}
   mx0 = b1;
   mr = mr + mx0 * my1 (rnd);             {mr = u = b0 * v0 + b1 * v1}
   if mv sat mr;

   ay1 = mx1;                             {ay1 = x = input}
   ar = mr1 + ay1;                        {ar = y = x + u = output}

   dm(v1) = sr0;                          {update lowpass filter, v1 = v0}
   tapin(i2, m2, ar);                     {put y in tap-0}
   cdelay(i2, m2);                        {update delay}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = ar;                   {left output sample}
   dm(tx_buf + 2) = ar;                   {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
