{lowpass.dsp - lowpass reverb}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on lowpass.c of Introduction to Signal Processing.
 Sample processing algorithm from Eq.(8.2.34) and Fig.8.2.21:
       for each input x do:
           sD = tap(D, w, p, D)                  get D-th tap
           v0 = a * v1 + sD                      input to unit delay
           u = b0 * v0 + b1 * v1                 output of feedback filter
           y = x + u                             filter output
           v1 = v0                               update unit delay
           *p = y                                input to D-fold delay
           cdelay(D, w, &p)                      update D-fold delay
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.var/dm v1;                        {state of first-order feedback filter}

.const a  = 0x4000;                {a  = 0.50}
.const b0 = 0x199a;                {b0 = 0.20}
.const b1 = 0x0ccd;                {b1 = 0.10}

.const D = 3000;                   {TD = D/fs = 3/8 = 0.375 sec}
.var/dm/circ w[D+1];               {circular delay line}

i2 = ^w;  L2 = %w;                 {delay-line buffer pointer and length}

ax0 = 0;
dm(v1) = ax0;                      {clear feedback delay, v1 = 0}
zero(i2, m2, L2);                  {clear delay line}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);                  {left input sample}
   mx1 = dm(rx_buf + 2);                  {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   tap(i2, m2, D, mr1);                   {mr1 = sD = D-th tap}

   mx0 = a;
   my1 = dm(v1);                          {state v1}
   mr = mr + mx0 * my1 (rnd);             {mr = v0 = sD + a * v1}
   if mv sat mr;
   sr0 = mr1;                             {save v0 for later updating v1}

   my0 = b0;
   mr = mr1 * my0 (ss);                   {mr = b0 * v0}
   mx0 = b1;
   mr = mr + mx0 * my1 (rnd);             {mr = u = b0 * v0 + b1 * v1}
   if mv sat mr;

   ay1 = mx1;                             {ay1 = x = input}
   ar = mr1 + ay1;                        {ar = y = x + u = output}

   dm(v1) = sr0;                          {update lowpass filter, v1 = v0}
   tapin(i2, m2, ar);                     {put y in tap-0}
   cdelay(i2, m2);                        {update delay}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = ar;                   {left output sample}
   dm(tx_buf + 2) = ar;                   {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
