{multidel.dsp - multi-delay effects algorithm}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Fig. 8.2.27 of Introduction to Signal Processing.
 Assuming the feedback filters are plain multipliers, G1(z)=a1,
 G2(z)=a2, and circular buffers for the two delays, the sample 
 processing algorithm is:
     for each x do:
          s1D = tap(D1, w1, p1, D1)              D1-th tap output
          s2D = tap(D2, w2, p2, D2)              D2-th tap output
          y = b0 * x + b1 * s1D + b2 * s2D       filter output
          *p2 = s20 = s1D + a2 * s2D             put input s20 into delay-2
          cdelay(D2, w2, &p2)                    update delay-2
          *p1 = s10 = x + a1 * s1D               put input s10 into delay-1
          cdelay(D1, w1, &p1)                    update delay-1
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const a1 = 0x4000;                {a1 = 0.50}
.const a2 = 0x3333;                {a2 = 0.40}

.const b0 = 0x7fff;                {b0 = 0.9999, that is, 1}
.const b1 = 0x6666;                {b1 = 0.80}
.const b2 = 0x4ccd;                {b2 = 0.60}

.const D1 = 3000;
.const D2 = 4000;

.var/dm/circ w1[D1+1];             {delay-1 buffer}
.var/dm/circ w2[D2+1];             {delay-2 buffer}

i2 = ^w1; L2 = %w1;                {delay-line buffer pointer and length}
i3 = ^w2; L3 = %w2;                {delay-line buffer pointer and length}

zero(i2, m2, L2);                  {clear delay-1}
zero(i3, m3, L3);                  {clear delay-2}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   tap(i2, m2, D1, sr0);           {sr0 = s1D = D1-th tap}
   tap(i3, m3, D2, sr1);           {sr1 = s2D = D2-th tap}

   my0 = b0;
   mr = mx1 * my0 (ss);            {mr = b0 * x}

   my0 = b1;
   mr = mr + sr0 * my0 (ss);       {mr = b0 * x + b1 * s1D}

   my0 = b2;
   mr = mr + sr1 * my0 (rnd);      {mr = y = b0 * x + b1 * s1D + b2 * s2D}
   if mv sat mr;

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = mr1;           {left output sample}
   dm(tx_buf + 2) = mr1;           {right output sample}

{--- sample processing algorithm (cont'd) --------------------------------}

   mr = 0;
   mr1 = sr0;                      {mr = s1D}
   my0 = a2;
   mr = mr + sr1 * my0 (rnd);      {mr = s20 = s1D + a2 * s2D}
   if mv sat mr;
   tapin(i3, m3, mr1);             {put s20 into tap-0 of delay-2}
   cdelay(i3, m3);                 {update delay-2}

   mr = 0;
   mr1 = mx1;                      {mr = x}
   my0 = a1;
   mr = mr + sr0 * my0 (rnd);      {mr = s10 = x + a1 * s1D}
   if mv sat mr;
   tapin(i2, m2, mr1);             {put s10 into tap-0 of delay-1}
   cdelay(i2, m2);                 {update delay-1}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
