{notch.dsp - notching comb filter canceling 800 Hz harmonics}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Eq.(8.3.26) and Example 8.3.9 of Introd. to Signal Processing.
 Input signal is microphone input + 800 Hz square wave:

       x = A0 * (mike) + A * (square wave)

 To better avoid overflows, we use the transpose realization with
 sample processing algorithm:
       for each input x do:
              sD = tap(D, w, p, D)
              y = b * x + sD
              *p = s0 = a * y - b * x
              cdelay(D, w, &p)
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const A0 = 0x4000;                {A0 = 0.5 = input scale factor}

.const D = 10;                     {D = fs/f1 = period of square wave}
.var/dm/circ w[D+1];               {filter's delay-line buffer}

i2 = ^w; L2 = %w;                  {delay-line buffer pointer and length}

zero(i2, m2, L2);                  {clear delay line}

.const Ds = 800;                   {wavetable's min freq: fs/Ds = 10 Hz}
.var/dm/circ s[Ds];                {square wavetable}

i6 = ^s; L6 = %s;                  {wavetable pointer and length}

.const c = 80;                     {square wave frequency f1 = c * fs/Ds}
.const A = 0x1000;                 {A = A0/4 = square wave amplitude}

.init s: <squartbl.hex>;           {load wavetable}

.const a = 0x690d;                 {coefficient a = 0.8207 = 0x690d}
.const b = 0x7486;                 {coefficient b = 0.91035= 0x7486}
                                   {design specs: f1 = 800 Hz, Df = 50 Hz}
{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);                  {left input sample}
   mx1 = dm(rx_buf + 2);                  {right input sample}

{--- sample processing algorithm -----------------------------------------}

   wavgen(i6, m6, A, c, mr1);             {mr1 = A * square wave}

   my1 = A0;
   mr = mr + mx1 * my1 (rnd);             {mr = x = A0 * mike + A * square}
   if mv sat mr;

  {ar = mr1; jump nofilter;}              {uncomment to bypass filter}

   mx1 = mr1;                             {save x temporarily}

   my0 = b;
   mr = mx1 * my0 (rnd);                  {mr1 = b * x}
   if mv sat mr;

   tap(i2, m2, D, ay1);                   {ay1 = sD = D-th tap}
   ar = mr1 + ay1;                        {ar = y = b * x + sD = output}

   my1 = a;
   mr = ar * my1 (ss);                    {mr = a * y}
   my1 = b;
   mr = mr - mx1 * my1 (rnd);             {mr = s0 = a * y - b * x}
   if mv sat mr;

   tapin(i2, m2, mr1);                    {put s0 into tap-0}
   cdelay(i2, m2);                        {update delay line}

{--- write samples to codec ---------------------------------------------}

nofilter:

   dm(tx_buf + 1) = ar;                   {left output sample}
   dm(tx_buf + 2) = ar;                   {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
