{notch1.dsp - single-notch filter at 800 Hz & width 50 Hz - direct form I}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Designed with parmeq.m of Appendix D, with the MATLAB commands:
       GB = 1/sqrt(2); f0 = 800; fs = 8000; Df = 50;
       [b, a, beta] = parmeq(1, 0, GB, 2*pi*f0/fs, 2*pi*Df/fs);
}


{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const A0 = 0x4000;                {A0 = 0.5 = input scale factor}

.const M = 2;                      {filter order}
.var/dm/circ w[M+1];               {y-delay-line buffer in DM}
.var/dm/circ v[M+1];               {x-delay-line buffer in DM}
.var/pm/circ a[M+1], b[M+1];       {concatenated coeffs in PM}

.const ea = 1;                     {scaling exponent for a}
.const eb = 1;                     {scaling exponent for b}
.const ex = 0;                     {input scaling exponent}

i2 = ^w; L2 = %w;                  {y-delay-line pointer}
i3 = ^v; L3 = %v;                  {x-delay-line pointer}
i4 = ^a; L4 = 2*(M+1);             {double-length a,b coefficients}

zero(i2, m2, L2);                  {clear y-delay line}
zero(i3, m3, L3);                  {clear x-delay line}

.const Ds = 800;                   {wavetable's min freq: fs/Ds = 10 Hz}
.var/dm/circ s[Ds];                {square wavetable}

i6 = ^s; L6 = %s;                  {wavetable pointer and length}

.const c = 80;                     {square wave freq: f1 = c * fs/Ds}
.const A = 0x1000;                 {A = A0/4 = square wave amplitude}

.init s: <squartbl.hex>;           {load wavetable}
.init a: <a.hex>;                  {denominator coefficients}
.init b: <b.hex>;                  {numerator coefficients}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);                  {left input sample}
   mx1 = dm(rx_buf + 2);                  {right input sample}

{--- sample processing algorithm -----------------------------------------}

   wavgen(i6, m6, A, c, mr1);             {mr1 = A * square wave}

   my1 = A0;
   mr = mr + mx1 * my1 (rnd);             {mr = x = A0 * mike + A * square}
   if mv sat mr;

  {sr1 = mr1; jump nofilter;}             {uncomment to bypass filter}
              
   sr = ashift mr1 by -ex (hi);                         {pre-scale x down}
                                                        {compute output y}
   cdir(M, i4, m4, i2, m2, i3, m3, ea, eb, sr1);        {input from sr1}
                                                        {output in ar}
   sr = ashift ar by ex (hi);                           {post-scale y up}

{--- write samples to codec ---------------------------------------------}

nofilter:

   dm(tx_buf + 1) = sr1;                   {left output sample}
   dm(tx_buf + 2) = sr1;                   {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
