{plain.dsp - plain reverb - IIR comb filter}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on plain.c of Introduction to Signal Processing.
 Sample processing algorithm from Eq.(8.2.14) and Fig.8.2.6:
       for each x do :
           sD = tap(D, w, p, D)                  get D-th tap
           y = x + a * sD                        filter output
           *p = y                                put y into tap-0
           cdelay(D, w, &p)                      update delay
}

{--- define sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const a = 0x4000;                 {feedback coefficient a = 0.50}
.const D = 3000;                   {TD = D/fs = 3/8 = 0.375 sec}
.var/dm/circ w[D+1];               {delay-line buffer}

i2 = ^w;  L2 = %w;                 {delay-line buffer pointer and length}

zero(i2, m2, L2);                  {clear delay line}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   ax1 = dm(rx_buf + 1);           {left input sample}
   mx1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm --- process right channel only ----------}

   mr = 0;
   mr1 = mx1;                      {mr = x = input}

   my1 = a;
   tap(i2, m2, D, mx1);            {mx1 = sD = D-th tap}
   mr = mr + mx1 * my1 (rnd);      {mr = x + a * sD}
   
   tapin(i2, m2, mr1);             {put y in tap-0}
   cdelay(i2, m2);                 {update delay}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = mr1;           {left output sample}
   dm(tx_buf + 2) = mr1;           {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
