{dnsample.dsp - downsampling and aliasing by decimation}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{--- choose sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc85c; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const M = 6;                      {decimation ratio; fs_low = fs/M}
.var/dm/circ s[M];                 {low-rate sampling pulse train}
.const A = 0x7fff;                 {A=1, amplitude of sampling pulse}

i5 = ^s; L5 = %s; m5 = 1;          {pointer to low-rate sampling pulse train}

cntr = M;                          {initialize pulse train to zero}
do zero_s until ce;                {s = [0, 0, ..., 0]}
zero_s:  dm(i5, m5) = 0;           {alternatively, may use zero(i5, m5, L5)}

ax0 = A;                           {set s[0] = A}
dm(s) = ax0;                       {s = [A, 0, ..., 0] = A and M-1 zeros}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- read input samples from codec ---------------------------------------}

   mr0 = dm(rx_buf + 1);           {left input sample}
   mr1 = dm(rx_buf + 2);           {right input sample}

{--- sample processing algorithm -----------------------------------------}

   ar = dm(i5, m5);                {get sampling pulse}
   ar = pass ar;                   {forces updating of AZ flag}
   if ne jump output;              {if AR != 0, write output}

   mr0 = 0;                        {else, write zero outputs}
   mr1 = 0;

{--- write output samples to codec ---------------------------------------}

   output:

#if 1                                            {optional amplify}
   sr = ashift mr0 by 1 (hi); mr0 = sr1;
   sr = ashift mr1 by 1 (hi); mr1 = sr1;
#endif

   dm(tx_buf + 1) = mr0;                         {left output sample}
   dm(tx_buf + 2) = mr1;                         {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
