{fm.dsp - FM modulation of using common sinusoidal wavetable}
{Junior DSP Lab - Rutgers ECE Dept - S. J. Orfanidis - Jan 1996}

{Based on Sec.8.1.3 of Introduction to Signal Processing}

{--- choose sampling rate in kHz: ----------------------------------------}
                    {0xc850 = 8      | 0xc851 = 5.5125   | 0xc852 = 16    }
                    {0xc853 = 11.025 | 0xc854 = 27.42857 | 0xc855 = 18.9  }
.const fs = 0xc850; {0xc856 = 32     | 0xc857 = 22.05    | 0xc859 = 37.8  }
                    {0xc85b = 44.1   | 0xc85c = 48       | 0xc85d = 33.075}
                    {0xc85e = 9.6    | 0xc85f = 6.615                     }
{-------------------------------------------------------------------------}

.include <c:\adi_dsp\macros\begin.dsp>;    {initializations and DSP macros}

{--- define constants, variables, and buffers ----------------------------}

.const D = 4000;
.var/dm/circ sine[D];              {min frequency f1 = fs/D = 8/4 = 2 Hz}

.init sine: <sinetbl.hex>;         {load one period of the wavetable}
                                   {generated by sinetbl.c}
.const Am = 30;                    {Am = 0.30 * c0}
.const cm = 1;                     {fm = cm * fs/D = 2 Hz}

i5 = ^sine; L5 = %sine;            {pointer for frequency generator}

.const A = 0x4000;                 {A = 0.50 = signal amplitude}
.const c0 = 100;                   {signal freq f0 = c0 * f1 = 200 Hz}

i6 = ^sine; L6 = %sine;            {pointer for signal generator}

{--- start processing input samples --------------------------------------}

wait: idle; jump wait;             {wait for interrupt and loop forever}
                                   {interrupt service routine starts here}
input_samples: ena sec_reg;        {enable secondary register set}

{--- sample processing algorithm -----------------------------------------}

   wavgen(i5, m5, Am, cm, ay1);    {ay1 = Am * sin(2*pi*fm*t)}
   ax1 = c0;
   ar = ax1 + ay1;                 {ar = c = c0 + Am * sin(2*pi*fm*t)}
   wavgen(i6, m6, A, ar, mx1);     {mx1 = FM-modulated sinusoid}

{--- write output samples to codec ---------------------------------------}

   dm(tx_buf + 1) = mx1;           {left output sample}
   dm(tx_buf + 2) = mx1;           {right output sample}

{--- return from interrupt -----------------------------------------------}

   rti;

.include <c:\adi_dsp\macros\end.dsp>;     {wrapup}
